%%%
%%% Author:
%%%   Andreas Franke (afranke@ags.uni-sb.de)
%%%
%%% Copyright:
%%%   Andreas Franke, 2000
%%%
%%% Last change:
%%%   $Date: 2000/05/06 13:08:08 $ by $Author: nilsf $
%%%   $Revision: 1.3 $
%%%
%%% See the file "LICENSE" or
%%%    http://mozart.ps.uni-sb.de/LICENSE.html
%%% for information on usage and redistribution
%%% of this file, and for a DISCLAIMER OF ALL
%%% WARRANTIES.
%%%

/**
% Fontify Oz source-files and export them as HTML
*/

functor
   
import
   EmacsFontifier          at 'x-oz://contrib/doc/code/Fontifier.ozf'
\ifndef OZH
   HTML('class':Html file) at 'x-ozlib://mathweb/html/Html.ozf'
\else
   HTML('class':Html file) at 'Html.ozf'
\endif
export
   'class': Writer   
   processFile:  FontifyFile
   processFiles: FontifyFiles
   
define

   %%
   %% compute file positions
   %%
   local
      FirstLine = 1 %% or 0 ???
      FirstCol  = 1 %% or 0 ???
   in
      class PosManager
         attr
            file: ''
            line: ~1
            col:  ~1
            
         meth posInit(FileNameA<='')
            file <- FileNameA
            line <- FirstLine
            col  <- FirstCol
         end
         meth getPos($)
            pos(@file @line @col)
         end
         meth incLine(N<=1)
            line <- @line + N
            col  <- FirstCol
         end
         meth incCol(N<=1)
            col  <- @col + N
         end
         meth updateC(C)
            if C==&\n
            then PosManager, incLine
            else PosManager, incCol
            end
         end
         meth updateS(S)
            case S of nil then skip
            [] C|Sr then
               PosManager,updateC(C)
               PosManager,updateS(Sr)
            end
         end
      end
   end
   
   FaceMap = unit(comment:   {ByteString.make "comment"}
                  keyword:   {ByteString.make "keyword"}
                  string:    {ByteString.make "string"}   
                  function:  {ByteString.make "functionname"}
                  type:      {ByteString.make "type"}
                  variable:  {ByteString.make "variablename"}
                  reference: {ByteString.make "reference"} % incl. constant
                  builtin:   {ByteString.make "builtin"}
                  %%warning: {ByteString.make "warning"}
                 )

   %% \t -> &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;
   %%    -> &nbsp;
   %% <  -> &lt;
   %% >  -> &gt;
   %% &  -> &amp;
   fun {EscapeHtml S}
      case S of nil then nil
      [] C|R then
         case C
         of &   then &&|&n|&b|&s|&p|&;|{EscapeHtml R}
         [] &<  then &&|&l|&t|&;|{EscapeHtml R}
         [] &>  then &&|&g|&t|&;|{EscapeHtml R}
	 [] &&  then &&|&a|&m|&p|&;|{EscapeHtml R}
	 [] &\t then {EscapeHtml & |& |& |& |& |& |& |& |R}
         else C|{EscapeHtml R}
         end
      end
   end   
   
   fun {NotNewline C} C\=&\n end

   %%
   %% create an HTML fragment from fontifier pair-list 
   %%
   class Writer from PosManager Html
                   
      meth writeText(S)
         S1 S2 in
         {List.takeDropWhile S NotNewline ?S1 ?S2}
         Writer, writeChars(S1)
         case S2 of nil then skip
         [] &\n|R then
            Writer, writeNewline
            Writer, writeText(R)
         end
      end
      meth writeChars(S)
         {self write(vs:{EscapeHtml S})}
         PosManager, incCol({Length S}) 
      end
      meth writeNewline
         {self write(vs:"<br>\n")}
         PosManager, incLine
         Writer, writePosLine
      end
      %%
      meth writePosLine
         Pos = PosManager, getPos($) in
         Writer, writeName(line#Pos.2)
      end
      meth writeName(Name)
         Html, tag(a(name:{ByteString.make Name}))
      end
      meth writeFace(Face S)
         if {HasFeature FaceMap Face} then
            %% begin special code
            case Face
            of function  then Writer, writeName(S)
            [] type      then Writer, writeName(S)
            [] reference then Writer, writeName(S)
            [] variable  then Writer, writeName(S)
            else skip
            end
            %% end special code
            Writer, writeSpan(FaceMap.Face S)
         elsecase Face
         of text then Writer, writeText(S)
         [] warning then raise warning({ByteString.make S}) end
         else raise unknownFace(Face {ByteString.make S}) end
         end
      end
      meth writeSpan(C S)
         Html, tag(span('class':C 'self'(writeText(S))))
      end
      meth WriteAll(L)
         case L of nil then skip
         [] (Face#S)|R then
            Writer, writeFace(Face S)
            Writer, WriteAll(R)
         end
      end
      meth writeCode(L)
         PosManager, posInit
         Writer, writePosLine
         Writer, WriteAll(L)
      end
   end
 
   %%
   %% predefined abstraction(s)
   %%
    
   class HtmlFile from HTML.file Writer end
   
   proc {SaveResult Link#Name OrigFile Result OutputFile}
      F = {New HtmlFile init(name:OutputFile flags:[write create truncate])} in
      {F writeDoc(title:   OrigFile
                  style:   "ozdoc.css"
                  margin:  a(href:Link Name)
                  heading: h3(code(OrigFile))
                  content: code('self'(writeCode(Result)))
                 )}
      {F close}
   end
   
   proc {FontifyFile FunctorNode InputFile OutputFile}
      Result = {EmacsFontifier.processFile 'oz' InputFile} in
      {SaveResult FunctorNode#"Functor" InputFile Result OutputFile} 
   end
   
   proc {FontifyFiles IOs}
      OBJECT = {New EmacsFontifier.'class' init}
      Sync   = {NewCell unit}
   in
      {ForAll IOs
       proc {$ InputFile#OutputFile}
          OldSync NewSync Result in
          {OBJECT enqueueFile('oz' InputFile ?Result)}
          {Exchange Sync OldSync NewSync}
          thread
             {SaveResult InputFile#"Source File" InputFile Result OutputFile}
             OldSync = NewSync
          end
       end}
      {OBJECT synck}
      {Wait {Access Sync}}
   end

end




