%%%
%%% Authors:
%%%   Nils Franzen (nils.franzen@sics.se)
%%%   Erik Klintskog (erik.klintskog@sics.se)
%%%
%%% Copyright:
%%%   Nils Franzen, 1999
%%%   Erik Klintskog, 2000
%%%
%%% Last change:
%%%   $Date: 2000/11/15 15:24:55 $ by $Author: nilsf $
%%%   $Revision: 1.8 $
%%%
%%% See the file "LICENSE" or
%%%    http://mozart.ps.uni-sb.de/LICENSE.html
%%% for information on usage and redistribution
%%% of this file, and for a DISCLAIMER OF ALL
%%% WARRANTIES.
%%%

functor
import
   Tk
   System(show:Show printInfo)
   Browser(browse:Browse)
   CanvasNode
   AnimateNodes
   OS
export
   Display
define
   proc{While P Do}
      if {P} then {Do} {While P Do} end
   end
   local
      FTI=Float.toInt
      ITF=Int.toFloat
   in
      fun{Dist P1 P2}
	 {FTI {Sqrt {ITF {Pow (P1.1 - P2.1) 2} + {Pow (P1.2 - P2.2) 2}}}}
      end
   end

   proc{SavePostscript DC}
      case {Tk.return tk_getSaveFile(title:"Saving postscript..."
				     parent:DC
				     initialfile:"functormap.ps"
				     filetypes:q(q('Ps' q('.ps'))
						 q('All Files' '*')
						)
				    )}
      of nil then skip
      elseof File then [X0 Y0 X1 Y1]={DC tkReturnListInt(bbox alltag $)} in
	 {DC tk(postscript file:File x:X0 y:Y0 width:X1-X0 height:Y1-Y0 colormode:color)}
      end
   end
   proc{Display DFs Ignores}
      Params={NewCell parameters(friction:0.5
				 maxStep:40.0
				 gravity:~3000.0
				 spring:0.01
				 strong:linear
				)}

      ParamInfo=info(friction:float(0.0#1.0)
		     maxStep:float(2.0#100.0)
		     gravity:float(~10000.0#10000.0)
		     spring:float(0.0#1.0)
		     strong:list(linear square))
      
      proc{RefineLayout}
	 {DC tk(delete linetag)}
	 {StartLayout}
      end
      proc{StartLayout}
	 Os=AllNodes
      in
	 %% StartLayout requires that InitLayout has been called
	 try 
	    Para={Access Params}
	 in
	    {For 1 1000 1 proc{$ _}
			     Move={AnimateNodes.animate Os nil nil Para}
			  in
			     {System.printInfo {Length Move}#","}
			     {ForAll Move proc{$ M}
					     RC = {M getRealCoord($)} in
					     {M setCoord(RC)}
					     {M moveTo(x:{FloatToInt RC.1}
						       y:{FloatToInt RC.2})}
					  end}
			     if Move==nil then raise done end end
			  end} 
	 catch done then skip
	 [] X then {Show X}
	 end
	 {System.printInfo "\n"}

	 %% Connect all import nodes
	 {ForAll Os proc{$ N} {N drawConnections} end}
	 
	 %% Configure scrollregion
	 {SetScrollRegion}
      end

      DONE

      %% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% Graphics
      T={New Tk.toplevel tkInit(title:"Functor Map" delete:proc{$} {T tkClose} DONE=unit end)}
      DC={New Tk.canvas tkInit(parent:T bg:white width:500 height:600)}
      SX={New Tk.scrollbar tkInit(parent:T orient:horizontal width:8)}
      SY={New Tk.scrollbar tkInit(parent:T orient:vertical width:8)}
      {Tk.addXScrollbar DC SX}
      {Tk.addYScrollbar DC SY}
      BF={New Tk.frame tkInit(parent:T)}
      B1={New Tk.button tkInit(parent:BF text:"Refine Layout" action:RefineLayout)}
      B2={New Tk.button tkInit(parent:BF text:"Save Postscript" action:SavePostscript#DC)}
      B3={New Tk.button tkInit(parent:BF text:"Quit" action:proc{$} {T tkClose} DONE=unit end)}
      {Tk.batch [grid(DC row:0 column:0 sticky:news)
		 grid(SX row:1 column:0 sticky:we)
		 grid(SY row:0 column:2 sticky:ns)
		 grid(BF row:2 column:0 columnspan:3 sticky:we)
		 grid(B1 row:0 column:0 sticky:we)
		 grid(B2 row:0 column:1 sticky:we)
		 grid(B3 row:0 column:2 sticky:we)
		 grid(columnconfigure T 0 weight:1)
		 grid(rowconfigure T 0 weight:1)
		 grid(columnconfigure BF 0 weight:1)
		 grid(columnconfigure BF 1 weight:1)		 
		 grid(columnconfigure BF 2 weight:1)]}

      proc{SetScrollRegion} [X0 Y0 X1 Y1]={DC tkReturnListInt(bbox alltag $)} in
	 {DC tk(configure scrollregion:q(X0-5 Y0-5 X1+5 Y1+5))}
      end

      %%% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% Box class
      class Box from CanvasNode.node
	 feat tag f imps 
	 attr x y dx dy imp impby:nil imp2
	 meth init(F text:T x:X y:Y) T1={New Tk.canvasTag tkInit(parent:DC)} 
	    %% Create functor title
	    {DC tk(crea text X Y text:T fill:black tags:T1)}
	    %% Box containing then load
	    [X0 Y0 X1 Y1]={DC tkReturnListInt(bbox T1 $)}
	    T2={New Tk.canvasTag tkInit(parent:DC)}
	    DX=4 DY=3 S=3
	 in
	    %% Shadow
	    {DC tk(crea rect X0-DX+S Y0-DY+S X1+DX+S Y1+DY+S
		   width:0
		   outline:nil
		   fill:grey
		   stipple:gray50
		   tags:T2)}
	    %% Box
	    {DC tk(crea rect X0-DX Y0-DY X1+DX Y1+DY
		   fill:wheat
		   outline:black
		   width:1
		   tags:q(alltag T2))}
	 
	    {DC tk(lower T2 T1)}
	    {DC tk(addtag T1 withtag T2)}
	 
	    x<-X y<-Y
	    self.f=F
	    self.tag=T1
	    imp<-{Filter {Map {Append F.'import' F.'require'}
			  fun{$ X} {String.toAtom X.filename} end}
		  fun{$ X} X\='' andthen {Not {Member X Ignores}} end} 
	    self.imps={Length @imp}
	    {T1 tkBind(event:'<3>'
		       action:proc{$}
				 L=self.f.name
			      in
				 {Browse L(x:@x y:@y imp:@imp impby:@impby links:{self getLinks($)})}
			      end)}

	    %% Bind movement of nodes
	    {T1 tkBind(event:'<Button-1>'
		       args:[int(x) int(y)]
		       action:proc{$ CX CY}
				 X={DC tkReturnInt(canvasx CX $)}
				 Y={DC tkReturnInt(canvasy CY $)}
			      in
				 @dx=@x-X
				 @dy=@y-Y
			      end)}
      
	    {T1 tkBind(event:'<B1-Motion>'
		       args: [int(x) int(y)]
		       action:proc{$ CX CY}
				 X={DC tkReturnInt(canvasx CX $)}
				 Y={DC tkReturnInt(canvasy CY $)}
				 OX=@x
				 OY=@y
			      in
				 x<-OX+(X-OX+@dx)
				 y<-OY+(Y-OY+@dy)
				 {self.tag tk(move @x-OX @y-OY)}
			      end)}
      
	    {T1 tkBind(event:'<ButtonRelease-1>'
		       action:proc{$}
				 dx<-_
				 dy<-_
				 {DC tk(delete linetag)}
				 {ForAll AllNodes proc{$ N}
						     {N drawConnections}
						     {N updateInternal}
						  end}
			      end)}
	 end
	 meth setMassCentra(S)
	    [X0 _ X1 _]={DC tkReturnListInt(bbox self.tag $)}
	    R = X1-X0 div 2
	 in
	    {self setMassCentras({Loop.forThread 0 R S
				  fun{$ Acc N}
				     {IntToFloat N}#0.0|~{IntToFloat N}#0.0|Acc
				  end nil})}
	    
	 end
	 
	 meth Move(dx:DX dy:DY) {self.tag tk(move DX DY)} end
	 meth moveTo(x:X y:Y)
	    {self Move(dx:X-@x dy:Y-@y)}
	    x<-X y<-Y
	 end
	 meth updateInternal
	    XY={IntToFloat @x}#{IntToFloat @y}
	 in
	    {self setRealCoord(XY)}
	    {self setCoord(XY)}
	 end
	 meth setImportedBy O N in
	    O=imp<-N
	    N={Map O fun{$ X} {Dictionary.get NodeDB X} end}
	    {ForAll N proc{$ X}
			 if {Not {Member X.f.file Ignores}} then
			    {X importedBy(self)}
			    {self setLinks(X|{self getLinks($)})}
			 end
		      end}
	 end
	 meth importedBy(S) O N in
	    O=impby<-N
	    N=S|O
	 end
	 meth getXY($) @x#@y end
	 meth drawConnections MXYC={self getXY($)} in
	    {ForAll @imp proc{$ N}
			    XY1={N getCorner(MXYC $ type:'in')}
			    MXY={self getCorner(XY1 $ type:out)}
			    XY={N getCorner(MXY $ type:'in')}
			 in
			    {DC tk(crea line MXY.1 MXY.2 XY.1 XY.2 fill:red width:1 arrow:last tags:linetag)}
			 end}
	 end
	 meth getCorner(XY C type:T<=out)
	    [DX DY]=if T==out then [0 0] else [2 2] end
	    [X0 Y0 X1 Y1]={DC tkReturnListInt(bbox self.tag $)}
	    Mx=X0+((X1-X0) div 2)
	    My=Y0+((Y1-Y0) div 2)
	    MMx=(X1-X0) div 4
	    AllCs=[X0-DX#Y0-DY Mx-MMx#Y0-DY Mx#Y0-DY Mx+MMx#Y0-DY X1+DX#Y1-DY
		   X0-DX#My X1+DX#My
		   X0-DX#Y1+DY Mx-MMx#Y1+DY Mx#Y1+DY Mx+MMx#Y1+DY X1+DX#Y1+DY]
	    C1={FoldR AllCs.2 fun{$ N O} D={Dist XY N} in
				 if D<O.dist then d(N dist:D) else O end
			      end d(AllCs.1 dist:{Dist XY AllCs.1})}
	 in C=C1.1 end
	 meth getLinesCoords($) P={self getXY($)} in {Map @imp2 fun{$ X} P#{X getXY($)} end} end
      end
      NodeDB={NewDictionary}
      AllNodes={Map {Filter DFs
		     fun{$ F}
			{Not {Member F.file Ignores}}
		     end}
		proc{$ F A} 
		   A={New Box init(F text:F.name x:0 y:0)}
		   {Dictionary.put NodeDB F.file A}
		end}
   in
      {ForAll AllNodes proc{$ X} {X setImportedBy} end}

      %% Display all nodes at random positions
      {List.forAllInd AllNodes proc{$ I X}
				  Ay= I * 10
				  Ax= I * {OS.rand} mod 471
				  AXY={IntToFloat Ax}#{IntToFloat Ay}
			       in
				  {X moveTo(x: Ax y: Ay)}
				  {X setRealCoord(AXY)}
				  {X setCoord(AXY)}
			       end}
      {SetScrollRegion}
      
      {StartLayout}
      {Wait DONE}
   end
in
   skip
end


