%%%
%%% Authors:
%%%   Nils Franzen (nils.franzen@sics.se)
%%%   Andreas Sundstrom (andreas@sics.se)
%%%
%%% Copyright:
%%%   Nils Franzen, 1999
%%%   Andreas Sundstrom, 1999
%%%
%%% Last change:
%%%   $Date: 2000/10/05 12:21:39 $ by $Author: annan $
%%%   $Revision: 1.12 $
%%%
%%% See the file "LICENSE" or
%%%    http://mozart.ps.uni-sb.de/LICENSE.html
%%% for information on usage and redistribution
%%% of this file, and for a DISCLAIMER OF ALL
%%% WARRANTIES.
%%%

/**
% Reads command line arguments and check if specified out directory
% exists (else create it!). Calls Ozh.analyze with all command-line arguments.
*/

functor
import
   OS(system stat)
   System(printError showInfo)
   Property(get)
   Application(getCmdArgs exit)
\ifdef OZH
   Ozh % Use this case when generating ozh documentation for this file
\else
   Ozh at 'x-ozlib://mozart/ozh/Ozh.ozf'
\endif
define
    VERSION=
    \insert VERSION
    Mkdir=if {Property.get 'platform.os' $}=='win32' then "mkdir " else "mkdir -p " end
    Spec=record('out'(single char:&o type:string default:nil)
		'stylesheet'(single char:&s type:string default:nil)
		'index'(single char:&i type:bool default:true)
		'map'(single char:&m type:bool default:false)
		'mapignore'(multiple type:atom default:nil)
		'copy'(single char:&c type:bool default:true)
		'args'(single char:&a type:bool default:true)
		'dump'(single char:&d type:string default:nil)
		'help'(single char:&h type:bool default:false)
		'cvs'(single char:&C type:bool default:false)
		'classes'(single type:bool default:true)
		'fontify'(single char:&f type:bool default:false)
		'path'(multiple char:&P type:atom default:nil)
		'all'(single type:bool default:false)
		'version'(single char:&v type:bool default:false)
		'define'(multiple char:&D type:atom default:nil))

    proc{CheckDirectory D} A=try {OS.stat D}.type==dir catch _ then false end in
       if A==false then B=({OS.system Mkdir#D}==0) in
	  if B==false then
	     M=try
		  if {OS.stat D}.type\=dir then ", file exists!"
		  else ", try again! (maybe a tmp error)"
		  end
	       catch _ then ", permission denied!"
	       end
	  in
	     {System.showInfo "*** Can not create directory "#D#M#"\n*** Shutting down..."}
	     raise quit end
	  end
       end
    end
    proc{ShowHelp M}
       {System.printError
	if M==unit then nil else "Command line option error: "#M#"\n" end#
	"Usage: "#{Property.get 'application.url'}#" <File1> ... <FileN> [options]\n"#
	"   --out <Dir>          Alias: -o <Dir>  Destination directory\n"#
	"   --define <NAME>      Alias: -D <Name> Define macro name NAME\n"#
	"   --path <DIR>         Alias: -P <DIR>  Define alternative search path\n"#
	"   --stylesheet <URL>   Alias: -s <URL>  Use this stylesheet instead (overrides --copy)\n"#
	"   --version            Alias: -v  Display version\n"#
	"   --(no)cvs            Alias: -C  Parse CVS information in functor header\n"#
	"   --(no)fontify        Alias: -f  Generate a pretty print version of the source code\n"#
	"   --(no)all            Display everything defined in a functor (not only exported)\n"#
	"   --(no)classes      * View detailed description of each class\n"#
	"   --(no)copy         * Copy stylesheet to output directory\n"#
	"   --(no)index        * Create index file\n"#
	"   --(no)args         * Keep the name of the arguments\n"#
	"   --(no)map            Show relations between functors\n"#
	"   --mapignore <Funct>  Alias: -I <Funct> Ignore <Funct> when creating map\n"
       }
    end
 in
    try
       Args = {Application.getCmdArgs Spec}
       Dir=if Args.out\=nil andthen {List.last Args.out}\=&/ then Args.out#"/" else Args.out end
    in
       %% Ask for help?
       if Args.help==true then
	  {ShowHelp unit}
	  {Application.exit 0}
       end

       %% Ask for version
       if Args.version then
	  {System.showInfo "Ozh version "#VERSION}
	  {Application.exit 0}
       end
      
      if Dir\=nil then {CheckDirectory Dir} end
      case Args.1 of _|_ then
	 {Ozh.analyze {AdjoinAt Args out Dir}}
	 {Application.exit 0}
      elseof nil then
	 {ShowHelp "Expecting at least one input file!"}
	 {Application.exit 3}
      end
   catch X then
      case X of quit then
	 {Application.exit 0}
      elseof error(ap(usage M) ...) then
	 {ShowHelp M}
	 {Application.exit 2}
      elseof E then
	 raise E end
      end
   end
end

